{*--------------------------------------------------------------------------*}
{*  Color_App  1.3  -- add colors to your Application dialogs.              *}
{*        by Doug Hood.  CIS 70324,3336                                     *}
{*           Released to the PUBLIC DOMAIN.                                 *}
{*            (in a small effort to repay my debt to the PUBLIC Domain)     *}
{*--------------------------------------------------------------------------*}
{*    Version 1.3 adds support for BP 7.0 /TVision 2.0                      *}
{*      using conditional compilation for BOTH BP6.0 and BP7.0 support      *}
{*--------------------------------------------------------------------------*}
{*    Version 1.2 adds Color_InputLine Beeping when FULL.                   *}
{*--------------------------------------------------------------------------*}
{*    Version 1.1 adds Color_StaticText2 (only the chars are colored)       *}
{*--------------------------------------------------------------------------*}
{*   To Use the COLOR features (Buttons, etc) you must:                     *}
{*     0: Have your application be a PCOLOR_APP from this unit!!!!          *}
{*     1: Have your DIALOG be a PCOLOR_DIALOG from this unit!               *}
{*     2: Pascal-Code the PCOLOR_BUTTONS/PCOLOR_STATICTEXT exactly          *}
{*        the same as if they were 'normal' TButton/TStaticText.            *}
{*     3: My 'FIELDS_C' also unit knows about these COLOR objects.          *}
{*--------------------------------------------------------------------------*}
{*--------------------------------------------------------------------------*}
{*  NOTE: i am taking advantage of the upper 8 colors by disabling the      *}
{*        ability to 'flash'                                                *}
{*--------------------------------------------------------------------------*}
UNIT Color_App;    {Colored Application Buttons/TEXT!!}


INTERFACE
USES App,       {TVISION, modified version to handle COLOR buttons!}
     Dialogs,   {TVISION, button}
     Views,     {TVision, titlestr}
     Drivers,   {TEvent}
     Objects;   {TVision, misc}

CONST
  Button_Color_Default :byte = 0;     {green}
  Button_Black         :byte = 1;
  Button_Dk_Blue       :byte = 2;
  Button_Green         :byte = 3;
  Button_Cyan          :byte = 4;
  Button_Red           :byte = 5;
  Button_Purple        :byte = 6;
  Button_Brown         :byte = 7;
  Button_Grey          :byte = 8;

  Button_Dk_Grey       :byte = 9;
  Button_Lt_Blue       :byte = 10;
  Button_Lt_Green      :byte = 11;
  Button_Lt_Cyan       :byte = 12;
  Button_Lt_Red        :byte = 13;
  Button_Lt_Purple     :byte = 14;
  Button_Yellow        :byte = 15;
  Button_White         :byte = 16;

{*************************************************************************}
{*  TAPPLICATION (TVision)                                               *}
{*    Method:GetPalette                                                  *}
{*        |                                                              *}
{*        +Color_App                                                     *}
{*           Method:GetPalette                                           *}
{*           Method:HandleEvent (beep when full)                         *}
{*                                                                       *}
{*                                                                       *}
{*  TDIALOG (Tvision)                                                    *}
{*    Method:Init         (turn off blink ability to get 8 more colors)  *}
{*    Method:GetPalette                                                  *}
{*        |                                                              *}
{*        + Color_Dialog (in unit Color_App)                             *}
{*           Method:GetPalette                                           *}
{*                                                                       *}
{*                                                                       *}
{*  TBUTTON (TVision)                                                    *}
{*    Constructor:Init                                                   *}
{*    Method:GetPalette                                                  *}
{*        |                                                              *}
{*        + Color_Button (in unit Color_App)                             *}
{*           Constructor:Init   (add color param)                        *}
{*           Method:GetPalette                                           *}
{*************************************************************************}


TYPE
  PColor_InputLine = ^Color_Inputline;
  Color_InputLine = OBJECT(TInputLine)
    constructor Init (var Bounds: OBJECTS.TRect; AMaxLen: Integer;
                      Button_Color_ID : byte);   {new parameter!}
    function GetPalette : PPalette; virtual;     {MUST Override!}
    procedure HandleEvent (var Event : TEvent); virtual;
                                               {beep when too long}
  private
    Color         : word;
  end; {color_inputline}


  PColor_Button = ^Color_Button;
  Color_Button = object(DIALOGS.TButton)
    constructor Init (var Bounds      : OBJECTS.TRect;
                      ATitle          : VIEWS.TTitleStr;
                      ACommand        : Word;
                      AFlags          : Byte;
                      Button_Color_ID : byte);   {new parameter!}
    function GetPalette : PPalette; virtual;     {MUST Override!}
    procedure New_Color (Button_Color_ID : Byte);
    function Get_Color : integer;
    procedure Hide_Shadow;
  private
    Color         : word;
    Shadow_Hidden : boolean;
  end; {Color_Button}

  {*-----------------------------------------------------------------*}
  {* The background, not the chars is colored.                       *}
  {*-----------------------------------------------------------------*}
  PColor_StaticText = ^Color_StaticText;
  Color_StaticText = object(DIALOGS.TStaticText)
    constructor Init (var Bounds      : OBJECTS.TRect;
                      AText           : string;
                      Button_Color_ID : byte);   {new parameter!}
    function GetPalette : PPalette; virtual;     {MUST Override!}
  private
    Color         : word;
  end; {Color_StaticText}

  {*-----------------------------------------------------------------*}
  {* Only the chars, not the background is colored                   *}
  {*-----------------------------------------------------------------*}
  PColor_StaticText2 = ^Color_StaticText2;
  Color_StaticText2 = object(Color_StaticText)
    function GetPalette : PPalette; virtual;     {MUST Override!}
  end; {Color_StaticText2}


  PColor_Dialog = ^Color_Dialog;
  Color_Dialog = object(TDialog)
    constructor init (var bounds: TRect; ATitle: TTitleStr);
    function GetPalette : PPalette; virtual;     {MUST Override!}
  end; {Color_Dialog}


  PColor_Application = ^Color_Application;
                 {this elimates need to tweak APP.pas}
  Color_Application = object(TApplication)
    function GetPalette : PPalette; virtual;     {MUST Override!}
  end; {Color_Application}

  {*-----------------------------------------------------------*}
  {* If you want blinking, you must call SetBlink(TRUE)        *}
  {* after a color dialog                                      *}
  {* NOTE: This should be called (FALSE) when returning        *}
  {*       from a DOS_SHELL. (since another program may        *}
  {*       haved changed the state behing my back)             *}
  {*-----------------------------------------------------------*}
  procedure SetBlink(State: Boolean);




{*************************************************************************}
{*************************************************************************}
{*************************************************************************}
IMPLEMENTATION


{*************************************************************************}
procedure SetBlink(State: Boolean); assembler; {by steve shafer}
asm
  mov ax,$1003
  mov bl,state
  push bp
  int $10
  pop bp
end; {setblink}


{**************************************************************************}
constructor Color_InputLine.Init;
begin
  TInputLine.Init (Bounds, AMaxLen);
  Color := Button_Color_ID;
end; {init}

{**************************************************************************}
function Color_InputLine.GetPalette : PPalette;
var
  D_Palette : string[4];
begin
        {#19#19#20#21;  => TVision default colors for TInputLine}
  D_Palette := TInputLine.GetPalette^;

  IF ((Color > 0) and (Color < 17)) THEN
    BEGIN
      D_Palette[1] := CHAR(30+(Color*3));     {normal}
      D_Palette[2] := D_Palette[1];
      D_Palette[3] := CHAR(30+(Color*3)+1);   {selected}
    END;

  GetPalette := @D_Palette;
end; {color_inputline.getpalette}

{***********************************************************************}
procedure Color_InputLine.HandleEvent(var Event: TEvent);
var
  c : integer;
begin
  c := ORD (Event.CharCode);
  IF ((Event.What = evKeyDown) and
      (c > 0) and
      (SelStart = 0) and (SelEnd = 0) and
      ((c >= 32) and (c <= 255)) and
      (LENGTH(Data^) = MaxLen)) THEN      {* beep when CANT add char *}
    BEGIN
      Write (Chr(7));
      ClearEvent (Event);
    END
  ELSE TInputLine.HandleEvent(Event);

end; {color_inputline.handleevent}

{**************************************************************************}
constructor Color_Button.Init;
begin
  TButton.Init (Bounds, ATitle, ACommand, AFlags);
  Color := Button_Color_ID;
  Shadow_Hidden := FALSE;
end; {color_button.init}

{**************************************************************************}
procedure Color_Button.New_Color;
begin
  Color := Button_Color_ID;
  Draw;  {display the change}
end; {new_color}

{**************************************************************************}
function Color_Button.Get_Color : integer;
begin
  Get_Color := Color;
end; {get_color}

{**************************************************************************}
procedure Color_Button.Hide_Shadow;
begin
  Shadow_Hidden := TRUE;
  Draw;  {display the change}
end; {hide_shadow}

{**************************************************************************}
function Color_Button.GetPalette : PPalette;
   {*  NOTE: Colors assumes use of COLOR Monitor!!!!  *}
 {  bckgrnd / letters
   49=NO SHADOW       (maybe want to instead use char[1]?)
  0 - black         4 - red           8 - dark gray     C - light red
  1 - blue          5 - magenta       9 - light blue    D - light magenta
  2 - green         6 - brown         A - light green   E - yellow
  3 - cyan          7 - light gray    B - light cyan    F - white
 }                                       {see pg TP6,107 on methodology}
var
  D_Palette : string[8];
begin
     {#10#11#12#13#14#14#14#15;  => TVISION default colors for TButton}
  D_Palette := TButton.GetPalette^;

  IF ((Color > 0) and (Color < 17)) THEN
    BEGIN
      D_Palette[1] := CHAR(30+(Color*3));     {normal}
      D_Palette[2] := D_Palette[1];           {normal for highlights}
      D_Palette[3] := CHAR(30+(Color*3)+1);   {selected button}

      D_Palette[5] := CHAR(30+(Color*3)+2);   {shortcut}
      D_Palette[6] := D_Palette[5];
      D_Palette[7] := D_Palette[6];
    END;

  IF (Shadow_Hidden)
    THEN D_Palette[8] := #88;  {set to EXTENDED set (background)}

  GetPalette := @D_Palette;
end; {color_button.getpalette}

{**************************************************************************}
constructor Color_StaticText.Init;
begin
  TStaticText.Init (Bounds, AText);
  Color := Button_Color_ID;
end; {init}

{**************************************************************************}
function Color_StaticText.GetPalette : PPalette;
var
  D_Palette : string[1];
begin
       {#6;  => TVISION default colors for TButton}
  D_Palette := TStaticText.GetPalette^; {TVision default colors for TStaticText}

  IF ((Color > 0) and (Color < 17))
    THEN D_Palette[1] := CHAR(30+(Color*3));     {normal}

  {*************************************************************************}
  {* NOTE: in TVision 2.0, Must handle the THREE different types of        *}
  {* dialog background colors. This is handled in Color_Dialog.GetPalette  *}
  {*************************************************************************}

  GetPalette := @D_Palette;
end; {color_staticText.getpalette}

{**************************************************************************}
function Color_StaticText2.GetPalette : PPalette;
var
  D_Palette : string[1];
begin
       {#6;  => TVISION default colors for TButton}
  D_Palette := TStaticText.GetPalette^; {TVision default colors for TStaticText}

  IF ((Color > 0) and (Color < 17))
    THEN D_Palette[1] := CHAR(80+Color);     {normal}

  {*************************************************************************}
  {* NOTE: in TVision 2.0, Must handle the THREE different types of        *}
  {* dialog background colors. This is handled in Color_Dialog.GetPalette  *}
  {*************************************************************************}

  GetPalette := @D_Palette;
end; {color_statictext2.getpalette}


{**************************************************************************}
constructor Color_Dialog.Init (var bounds: TRect; ATitle: TTitleStr);
begin
  TDialog.Init (bounds, ATitle);
  SetBlink(False);
end; {init}

{**************************************************************************}
function Color_Dialog.GetPalette : PPalette;
  (*#32#33#34#35#36#37#38#39#40#41#42#43#44#45#46#47#48#49#50#51#52#53#54+
    #55#56#57#58#59#60#61#62#63+  {end of default CDialog}
    {*--------------------------------------------------------*}
    {* positions 33..110 is mapped similar pg 230 CDialog into *}
    {* a MODIFIED APP.CColor!                                 *}
    {*--------------------------------------------------------*}
  *)
const
{$IFDEF Ver60}          {* TVision 1.0 *}
  New_Palette : string[64] = {32 slots,     starting at 64 = length(ccolor+1)}
      #64#65#66+
      #67#68#69+
      #70#71#72+
      #73#74#75+
      #76#77#78+
      #79#80#81+
      #82#83#84+
      #85#86#87+

      #88#89#90+     {dk gray}
      #91#92#93+     {lt blue}
      #94#95#96+     {lt green}
      #97#98#99+     {lt cyan}
      #100#101#102+  {lt red}
      #103#104#105+  {lt magenta}
      #106#107#108+  {yellow}
      #109#110#111+  {white}
      {81}
      #112+          {special static grey+colored letter}
      #113+
      #114+
      #115+
      #116+
      #117+
      #118+
      #119+          {also used with hide the shadow}
      #120+
      #121+
      #122+
      #123+
      #124+
      #125+
      #126+
      #127;

{$ELSE}                 {* TVision 2.0 *}
  New_Palette : string[64] = {32 slots,  start at 128=length(cAppColor)+1}
      #128#129#130+
      #131#132#133+
      #134#135#136+
      #137#138#139+
      #140#141#142+
      #143#144#145+
      #146#147#148+
      #149#150#151+

      #152#153#154+     {dk gray}
      #155#156#157+     {lt blue}
      #158#159#160+     {lt green}
      #161#162#163+     {lt cyan}
      #164#165#166+  {lt red}
      #167#168#169+  {lt magenta}
      #170#171#172+  {yellow}
      #173#174#175+  {white}
      {81}
      #176+          {special static [DEFAULT dpGreyDialog]+colored letter}
      #177+          {* is adjusted dyanamically for other dp colors *}
      #178+          {* so slots 192..223 are reserved for this! *}
      #179+
      #180+
      #181+
      #182+
      #183+          {also used with hide the shadow}
      #184+
      #185+
      #186+
      #187+
      #188+
      #189+
      #190+
      #191;

  Blue_Palette : string [16] =
      #192+          {* This is copied into the New_Paletee when the *}
      #193+          {* dialog.palette = Dialogs.dpBlueDialog  *}
      #194+          {* so slots 192..223 are reserved for this! *}
      #195+
      #196+
      #197+
      #198+
      #199+          {also used with hide the shadow}
      #200+
      #201+
      #202+
      #203+
      #204+
      #205+
      #206+
      #207;

  Cyan_Palette : string [16] =
      #208+          {* This is copied into the New_Paletee when the *}
      #209+          {* dialog.palette = Dialogs.dpCyanDialog  *}
      #210+          {* so slots 192..223 are reserved for this! *}
      #211+
      #212+
      #213+
      #214+
      #215+          {also used with hide the shadow}
      #216+
      #217+
      #218+
      #219+
      #220+
      #221+
      #222+
      #223;


{$ENDIF}
var
  D_Palette  : string [32+ 64]; {32+new}
{$IFNDEF Ver60}
  Adjusted_New_Palette : string [64];
{$ENDIF}

begin
{$IFDEF Ver60}
  D_Palette := TDialog.GetPalette^ + {TVision default colors for TDialog}
               New_Palette;

{$ELSE}                 {* TVision 2.0, must handle the THREE palettes! *}
  IF (Palette = DIALOGS.dpGrayDialog)  {* DEFAULT *}
    THEN D_Palette := TDialog.GetPalette^ + {TVision default colors for TDialog}
                      New_Palette
  ELSE
    BEGIN
      {*******************************************************************}
      {* Must Adjust the Color_StaticText2 background to be in sync with *}
      {* the Dialogs current Palette                                     *}
      {*******************************************************************}
      {* make copy since dont want CONST to keep chg thru dialogs!       *}
      {*******************************************************************}
      Adjusted_New_Palette := New_Palette;

      IF (Palette = DIALOGS.dpBlueDialog)
        THEN Move (Blue_Palette[1], Adjusted_New_Palette[49], 16)
        ELSE Move (Cyan_Palette[1], Adjusted_New_Palette[49], 16);

      D_Palette := TDialog.GetPalette^ + {TVision default colors for TDialog}
                   Adjusted_New_Palette;
    END;
{$ENDIF}

  GetPalette := @D_Palette;
end; {color_dialog.getpalette}

{**************************************************************************}
function Color_Application.GetPalette : PPalette;
  (*  #32#33#34#35#36#37#38#39#40#41#42#43#44#45#46#47#48#49#50#51#52#53#54+
    #55#56#57#58#59#60#61#62#63+  {end of default CDialog}
    {*--------------------------------------------------------*}
    {* positions 32..63 is mapped similar pg 230 CDialog into *}
    {* a MODIFIED APP.CColor!                                 *}
    {*--------------------------------------------------------*}
  *)
    {*----------------------------------------*}
    {* DWH Specific access for COLORS/TEXT!   *}
    {*  (since dont have MULTI-inheritance)   *}
    {*----------------------------------------*}
const
{$IFDEF Ver60}
  New_Palette : string[64] = {31}
{$ELSE}
  New_Palette : string[64+32] = {31 + 32 for 2*16 color new palettes}
{$ENDIF}
    #$07#$0F#$0E+
    #$17#$1F#$1E+
    #$20#$2F#$2E+
    #$30#$3F#$3E+
    #$40#$4F#$4E+
    #$50#$5F#$5E+
    #$60#$6F#$6E+
    #$70#$7F#$7E+

    #$80#$8F#$8E+    {dk gray}
    #$90#$9F#$9E+    {lt blue}
    #$A0#$AF#$AE+    {lt green}
    #$B0#$BF#$BE+    {lt cyan}
    #$C0#$CF#$CE+    {lt red}
    #$D0#$DF#$DE+    {lt magenta}
    #$E0#$E7#$E2+    {yellow}
    #$F0#$F7#$F2+    {white}
    #$70+            {special static text grey+colored letter}
    #$71+            {""}
    #$72+
    #$73+
    #$74+
    #$75+
    #$76+
    #$77+            {also hide the shadow}
    #$78+
    #$79+
    #$7A+
    #$7B+
    #$7C+
    #$7D+
    #$7E+
    #$7F
{$IFDEF Ver60}          {* TVision 1.0 *}
    ;
{$ELSE}                 {* TVision 2.0 additional dialog palettes *}
    +
    #$10+            {special static text dpBlueDialog+colored letter}
    #$11+            {""}
    #$12+
    #$13+
    #$14+
    #$15+
    #$16+
    #$17+            {also hide the shadow}
    #$18+
    #$19+
    #$1A+
    #$1B+
    #$1C+
    #$1D+
    #$1E+
    #$1F+

    #$30+            {special static text dpCyanDialog+colored letter}
    #$31+            {""}
    #$32+
    #$33+
    #$34+
    #$35+
    #$36+
    #$37+            {also hide the shadow}
    #$38+
    #$39+
    #$3A+
    #$3B+
    #$3C+
    #$3D+
    #$3E+
    #$3F;
{$ENDIF}

var
{$IFDEF Ver60}          {* TVision 1.0  63 = length(CColor)     *}
  D_Palette  : string [63+   64];    (* 64 = length of my new adjusts *)
{$ELSE}                 {* TVision 2.0  127=length(CAppColor)+1 *}
  D_Palette  : string [127+  64 + 32];      (* additional 32 for 2 new *)
                                            (* dialog palettes         *)
{$ENDIF}
begin
  D_Palette := TApplication.GetPalette^ +  {TVision default colors for TDialog}
               New_Palette;

  GetPalette := @D_Palette;
end; {color_application.getpalette}


END. {color_App}
